import {
  Entity,
  Column,
  PrimaryGeneratedColumn,
  ManyToOne,
  OneToMany,
  Timestamp,
  PrimaryColumn,
} from 'typeorm';
import { PersonEntity, PersonRoleEntity } from '../user/user.entity';
import { FactionEntity } from '../game/faction.entity';
import {
  CoordinateEntity,
  MapEntity,
  MapDrawingEntity,
  MapDrawingTypeEntity,
} from './coordinate.entity';

// table that stores all created games
@Entity('Game')
export class GameEntity {
  @PrimaryGeneratedColumn('uuid') id: string;
  @Column('text') name: string;
  @Column('text') desc: string;
  @Column('json') center: JSON;
  @Column('json') map: JSON;
  @Column('timestamp') startdate: Timestamp;
  @Column('timestamp') enddate: Timestamp;

  @ManyToOne(type => MapEntity, map => map.games)
  gamemap: MapEntity;
  @OneToMany(type => FactionEntity, faction => faction.game)
  factions: FactionEntity[];
  @OneToMany(type => Game_PersonEntity, game_persons => game_persons.game)
  game_persons: Game_PersonEntity[];
  @OneToMany(
    type => ObjectivePointEntity,
    objective_points => objective_points.game,
  )
  objective_points: ObjectivePointEntity[];

  gameObject() {
    const { id, name } = this;
    return { id, name };
  }
}

// table that stores players associated with particular game
@Entity('Game_Person')
export class Game_PersonEntity {
  @PrimaryGeneratedColumn('uuid') gameId: string;
  @ManyToOne(type => FactionEntity, faction => faction.game_persons)
  faction: FactionEntity;
  @ManyToOne(type => GameEntity, game => game.game_persons)
  game: GameEntity;
  @ManyToOne(type => PersonEntity, person => person.game_persons)
  person: PersonEntity;
  @ManyToOne(type => PersonRoleEntity, person_role => person_role.game_persons)
  person_role: PersonRoleEntity;
  @ManyToOne(
    type => CoordinateEntity,
    game_person_coordinates => game_person_coordinates.game_persons,
  )
  game_person_coordinates: CoordinateEntity;
}

@Entity('ObjectivePoint')
export class ObjectivePointEntity {
  @PrimaryGeneratedColumn('uuid') objectivePointId: string;
  @Column({ type: 'text' }) objectivePointDescription: string;
  @Column({ type: 'float' }) objectivePointMultiplier: number;

  @ManyToOne(
    type => CoordinateEntity,
    coordinate => coordinate.objective_points,
  )
  coordinate: CoordinateEntity;
  @ManyToOne(type => GameEntity, game => game.objective_points)
  game: GameEntity;
  @OneToMany(
    type => ObjectivePoint_HistoryEntity,
    op_history => op_history.objective_point,
  )
  op_history: Game_PersonEntity[];
}

@Entity('ObjectivePoint_History')
export class ObjectivePoint_HistoryEntity {
  @PrimaryGeneratedColumn('uuid') oP_HistoryId: string;
  @Column({ type: 'timestamp' }) oP_HistoryTimestamp: Timestamp;
  @Column({}) oP_HistoryStatus: number;

  @ManyToOne(
    type => ObjectivePointEntity,
    objective_point => objective_point.op_history,
  )
  objective_point: ObjectivePointEntity;
}

@Entity('PowerUp')
export class PowerUpEntity {
  @PrimaryGeneratedColumn('uuid') powerUpId: string;
  @Column({ type: 'text' }) powerUpName: string;
  @Column({ type: 'text' }) powerUpDescription: string;
  @Column({ type: 'int' }) amount: number;
  @Column({ type: 'time' }) cooldown: string;

  @OneToMany(type => FactionEntity, factions => factions.powerUps)
  factions: Faction_PowerUpEntity[];
}

@Entity('Faction_PowerUp')
export class Faction_PowerUpEntity {
  @PrimaryGeneratedColumn('uuid') faction_powerUpId: string;

  @ManyToOne(type => FactionEntity, faction => faction.powerUps)
  faction: FactionEntity;
  @ManyToOne(type => PowerUpEntity, powerUp => powerUp.factions)
  powerUp: PowerUpEntity;
  @OneToMany(type => FP_HistoryEntity, histories => histories.faction_PowerUp)
  histories: FP_HistoryEntity[];
}

@Entity('FP_History')
export class FP_HistoryEntity {
  @PrimaryGeneratedColumn('uuid') historyId: string;
  @Column({ type: 'timestamp' }) historyTimeStamp: Timestamp;

  @ManyToOne(
    type => Faction_PowerUpEntity,
    faction_PowerUp => faction_PowerUp.histories,
  )
  faction_PowerUp: Faction_PowerUpEntity;
}
@Entity('Score')
export class ScoreEntity {
  @PrimaryGeneratedColumn('uuid') scoreId: string;
  @Column({ type: 'float' }) score: number;
  @Column({ type: 'timestamp' }) scoreTimeStamp: Timestamp;

  @ManyToOne(type => FactionEntity, factionName => factionName.scores)
  faction: FactionEntity;
}

@Entity('Task')
export class TaskEntity {
  @PrimaryGeneratedColumn('uuid') taskId: string;
  @Column({ type: 'text' }) taskName: string;
  @Column({ type: 'text' }) taskDescription: string;
  @Column({ type: 'text' }) taskWinner: string;
  @Column({ type: 'bool' }) taskIsActive: boolean;

  @ManyToOne(type => FactionEntity, faction => faction.tasks)
  faction: FactionEntity;
  @ManyToOne(type => PersonEntity, person => person.tasks)
  person: PersonEntity;
}
