import { Injectable, Logger, HttpException, HttpStatus } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository, In } from 'typeorm';

import { GameEntity, Game_PersonEntity } from './game.entity';
import { FactionEntity } from './faction.entity';
import { GameDTO } from './game.dto';
import { PersonEntity } from '../user/user.entity';

@Injectable()
export class GameService {
  constructor(
    @InjectRepository(GameEntity)
    private gameRepository: Repository<GameEntity>,
    @InjectRepository(FactionEntity)
    private factionRepository: Repository<FactionEntity>,
    @InjectRepository(PersonEntity)
    private personRepository: Repository<PersonEntity>,
    @InjectRepository(Game_PersonEntity)
    private game_PersonRepository: Repository<Game_PersonEntity>,
  ) {}

  // create a new game
  async createNewGame(personId: string, gameData: GameDTO) {
    // checks if a game with the same name exists already
    const { name } = gameData;
    if (await this.gameRepository.findOne({ where: { name } })) {
      throw new HttpException('Game already exists', HttpStatus.BAD_REQUEST);
    }
    // else add the game to the database
    const game = await this.gameRepository.create({
      ...gameData,
      factions: gameData.factions,
    });
    await this.gameRepository.insert(game);
    return {
      message: 'New game added',
    };
  }

  // edit already created game
  async editGame(id: string, gameData: Partial<GameDTO>) {
    try {
      // update game entry in db
      let update = await this.gameRepository.create({
        ...gameData,
        factions: gameData.factions,
      });
      await this.gameRepository.update({ id }, update);
      // add the factions to db
      if (gameData.factions) {
        gameData.factions.map(async faction => {
          let name = await this.factionRepository.create({
            ...faction,
            game: gameData,
          });
          await this.factionRepository.insert(name);
        });
      }
      return {
        message: 'Game updated',
      };
    } catch (error) {
      console.log(error);
    }
  }

  async deleteGame(id) {
    // TODO: Delete factions from Faction table associated with the deleted game
    await this.gameRepository.delete({ id });
    return {
      message: 'Game deleted',
    };
  }

  // checks the password, creates an entry in GamePerson table with associated role&faction
  async joinGame(person, gameId, json) {
    const user = await this.personRepository.findOne({
      where: { id: person },
    });
    const game = await this.gameRepository.findOne({ where: { id: gameId } });

    //const index = game.passwords.indexOf(json.password);

    // create game_Person entry
    /*     const gamePerson = await this.game_PersonRepository.create({
      faction,
      gameId,
      person,
    });
    */
    return 'WIP';
  }

  // returns name and id of each game
  async listGames() {
    const games = await this.gameRepository.find({ relations: ['factions'] });
    return games.map(game => {
      return game.gameObject();
    });
  }

  // returns information about a game identified by id
  async returnGameInfo(id: string) {
    const game = await this.gameRepository.findOne({
      where: { id: id },
      relations: ['factions'],
    });
    return game;
  }
}
